@@ -76,6 +76,8 @@ ABOUT_TEXT = """
  └───────────────────────────────────────────────────────────────────────────┘
 """.format(ver=__version__)
 
+UPDATE_INTERVALL = 100  # in ms
+
 while True:
     h,l,s = [random.random() for x in range(3)]
     if 0.4 <= l <= 0.7:
@@ -298,15 +300,17 @@ class MainApplication(ttk.Frame):
         self.playing_track_id = None
 
         def update_player():
-            intervall = 100
-            if hasattr(self, "player") and self.player.is_playing:
-                self.player.update()
-                track = self.loaded_album.tracklist[self.selected_track_id]
-                time = self.player.time
-                self.playhead = 100 / track["streaminfo"]["duration"] * time
-                if self.player.audio_driver == "PulseAudioDriver":
-                    intervall = 10
-            self.after(intervall, update_player)
+            try:
+                if hasattr(self, "player") and self.player.is_playing:
+                    self.player.update()
+                    track = self.loaded_album.tracklist[self.selected_track_id]
+                    time = self.player.time
+                    if str(self.playpause_button["state"]) == "normal":
+                        playhead = 100 / track["streaminfo"]["duration"] * time
+                        self.playhead = playhead
+                self.after(UPDATE_INTERVALL, update_player)
+            except tk._tkinter.TclError:
+                pass
 
         update_player()
 
@@ -384,7 +388,7 @@ class MainApplication(ttk.Frame):
 
         if platform.system() != "Darwin":
             self.file_menu.add_command(label="Quit",
-                                       command=self.master.destroy,
+                                       command=self.quit,
                                        accelerator=f"{modifier}-Q")
             self.help_menu = tk.Menu(self.menubar, tearoff=False)
             self.menubar.add_cascade(menu=self.help_menu, label="Help")
@@ -395,7 +399,7 @@ class MainApplication(ttk.Frame):
 
         self.parent.bind("<F1>", lambda e: HelpDialogue(self.master))
         self.parent.bind(f"<{modifier}-q>",
-                         lambda e: self.master.destroy())
+                         lambda e: self.quit())
 
         self.parent["menu"] = self.menubar
 
@@ -540,6 +544,8 @@ class MainApplication(ttk.Frame):
         def increment_track(step):
             selected_track_id = self.selected_track_id + step
             if 0 <= selected_track_id < len(self.loaded_album.tracklist):
+                if str(self.playpause_button["state"]) == "disabled":
+                    return
                 play_next = False
                 if self.playing_track_id is not None:
                     self.pause()
@@ -568,12 +574,13 @@ class MainApplication(ttk.Frame):
 
         def increment_playhead(step):
             if self.loaded_album is not None:
+                if str(self.playpause_button["state"]) == "disabled":
+                    return
                 track = self.loaded_album.tracklist[self.selected_track_id]
                 new_playhead = self.playhead + step
                 pos = track["streaminfo"]["duration"] / 100 * new_playhead
-                if pos < track["streaminfo"]["duration"] - 2:
-                    self.playhead = new_playhead
-                    self.player.seek(pos)
+                self.playhead = new_playhead
+                self.player.seek(pos)
 
         self.parent.bind(f"<Right>", lambda e: increment_playhead(1))
         self.parent.bind("l", lambda e: increment_playhead(1))
@@ -582,6 +589,8 @@ class MainApplication(ttk.Frame):
 
         def seek_to_beginning(e):
             if self.loaded_album is not None:
+                if str(self.playpause_button["state"]) == "disabled":
+                    return
                 self.playhead = 0
                 self.player.seek(0.0)
 
@@ -634,13 +643,14 @@ class MainApplication(ttk.Frame):
 
         def set_playhead_from_mouseclick(e):
             if self.loaded_album is not None:
+                if str(self.playpause_button["state"]) == "disabled":
+                    return
                 slider = e.widget
                 new_playhead = e.x / slider.winfo_width() * slider["maximum"]
                 track = self.loaded_album.tracklist[self.selected_track_id]
                 pos = track["streaminfo"]["duration"] / 100 * new_playhead
-                if pos < track["streaminfo"]["duration"] - 2:
-                    self.playhead = new_playhead
-                    self.player.seek(pos)
+                self.playhead = new_playhead
+                self.player.seek(pos)
 
         self.playhead_slider.bind("<ButtonPress-1>", set_playhead_from_mouseclick)
         self.playhead_slider.bind("<B1-Motion>", set_playhead_from_mouseclick)
@@ -653,6 +663,8 @@ class MainApplication(ttk.Frame):
         self.volume_slider.bind("<B1-Motion>", set_volume_from_mouseclick)
 
         def clicked_treeitem(e):
+            if str(self.playpause_button["state"]) == "disabled":
+                return
             item_id = self.tree.identify('item', e.x, e.y)
             if item_id == "":
                 return
@@ -765,7 +777,7 @@ class MainApplication(ttk.Frame):
                 0, 0, image=self.canvas.image, anchor="nw")
         self.loaded_album = None
         if self.playing_track_id is not None:
-            self.playpause()
+            self.pause()
         for i in self.tree.get_children():
             self.tree.delete(i)
         self.remove_arrows()
@@ -827,6 +839,24 @@ class MainApplication(ttk.Frame):
 
         def next_gapless():
             if self.selected_track_id + 1 < len(self.loaded_album.tracklist):
+                self.playpause_button["state"] = "disabled"
+                track = self.loaded_album.tracklist[self.selected_track_id]
+                dur = track["streaminfo"]["duration"]
+                tickspeed = UPDATE_INTERVALL / 1000
+                pos = self.playhead / 100 * dur + tickspeed
+                start = time.time()
+                # Update GUI after running out of audio data
+                while True:
+                    current_time = time.time()
+                    if current_time - start >= dur - (pos - tickspeed):
+                        break
+                    self.playhead = 100 / dur * (pos + time.time() - start)
+                    self.player.update(tick_only=True)
+                    self.parent.update()
+                    sleep_time = tickspeed - (time.time() - current_time)
+                    if sleep_time > 0:
+                        time.sleep(sleep_time)
+                self.playpause_button["state"] = "normal"
                 tags = self.tree.item(str(self.selected_track_id), "tags")
                 tags = [x for x in tags if x != "bold"]
                 tags.append("normal")
@@ -866,20 +896,26 @@ class MainApplication(ttk.Frame):
                 self.playing_track_id = track_id
                 self.load_track()
             else:
+                self.playpause_button["state"] = "disabled"
                 track = self.loaded_album.tracklist[self.selected_track_id]
                 dur = track["streaminfo"]["duration"]
-                tickspeed = 0.1
-                if self.player.audio_driver == "PulseAudioDriver":
-                    tickspeed = 0.01
+                tickspeed = UPDATE_INTERVALL / 1000
                 pos = self.playhead / 100 * dur + tickspeed
                 start = time.time()
-                self.playpause_button["state"] = "disabled"
-                while time.time() - start < dur - pos:
+                # Update playhead after running out of audio data
+                while True:
+                    current_time = time.time()
+                    if current_time - start >= dur - pos:
+                        break
                     self.playhead = 100 / dur * (pos + time.time() - start)
+                    self.player.update(tick_only=True)
                     self.parent.update()
-                    time.sleep(tickspeed)
+                    sleep_time = tickspeed - (time.time() - current_time)
+                    if sleep_time > 0:
+                        time.sleep(sleep_time)
                 self.playpause_button["state"] = "normal"
-                self.pause()
+                if self.playing_track_id is not None:
+                    self.pause()
                 self.tree.selection_set(["0"])
                 self.selected_track_id = 0
                 self.load_track()
@@ -1089,6 +1125,10 @@ class MainApplication(ttk.Frame):
                 title = prefix + title
         self.parent.title(title)
 
+    def quit(self):
+        self.player = None
+        del self.player
+        self.parent.destroy()
 
 def run():
     if platform.system() == "Windows":
@@ -1118,6 +1158,7 @@ def run():
     root.minsize(root.winfo_width(), root.winfo_height())
     root.lift()
     root.focus_force()
+    root.protocol('WM_DELETE_WINDOW', app.quit)
 
     try:
         app.load_album(os.path.abspath(sys.argv[1]))
